bl_info = {
	"name": "Msp GL datafiles",
	"author": "Mikko Rasa",
	"location": "File > Export",
	"description": "Export Msp GL data",
	"category": "Import-Export" }

if "bpy" in locals():
	import imp
	for sub in "armature", "export_armature", "export_mesh", "export_object", "export_scene", "mesh", "outfile", "properties", "util":
		if sub in locals():
			imp.reload(locals()[sub])

import bpy
from bpy_extras.io_utils import ExportHelper

class ExportMspGLBase(ExportHelper):
	show_progress = bpy.props.BoolProperty(name="Show progress", description="Display progress indicator while exporting", default=True)

	def execute(self, context):
		exporter = self.create_exporter()
		self.prepare_exporter(exporter)
		exporter.export(context, self.filepath)
		return {"FINISHED"}

	def create_exporter(self):
		raise Exception("create_exporter must be overridden")

	def prepare_exporter(self, exporter):
		for k, v in self.as_keywords().items():
			setattr(exporter, k, v)

class ExportMspGLMeshBase(ExportMspGLBase):
	use_strips = bpy.props.BoolProperty(name="Use strips", description="Combine faces into triangle strips", default=True)
	use_degen_tris = bpy.props.BoolProperty(name="Use degen tris", description="Concatenate triangle strips with degenerate triangles", default=False)
	max_strip_len = bpy.props.IntProperty(name="Max strip length", description="Maximum length for a triangle strip", default=1024, min=4, max=16384)
	optimize_cache = bpy.props.BoolProperty(name="Optimize cache", description="Optimize element order for vertex cache", default=True)
	cache_size = bpy.props.IntProperty(name="Cache size", description="Simulated vertex cache size used in optimization", default=64, min=8, max=1024)
	export_lines = bpy.props.BoolProperty(name="Export lines", description="Export edges without faces as lines", default=False)
	export_uv = bpy.props.EnumProperty(name="Export UV", description="Export UV coordinates", default="UNIT0",
		items=(("NONE", "None", "No UV coordinates are exported"),
			("UNIT0", "Unit 0", "UV coordinates for unit 0 are exported"),
			("ALL", "All", "All UV coordinates are exported")))
	tbn_vecs = bpy.props.BoolProperty(name="TBN vectors", description="Compute tangent and binormal vectors for vertices", default=False)
	tbn_uvtex = bpy.props.StringProperty(name="TBN UV layer", description="UV layer to use as basis for TBN vectors", default="")
	compound = bpy.props.BoolProperty(name="Compound", description="Combine all selected objects into one for exporting", default=False)
	smoothing = bpy.props.EnumProperty(name="Smoothing", description="Smoothing method to use", default="MSPGL",
		items=(("NONE", "None", "No smoothing"),
			("BLENDER", "Blender", "Use Blender's vertex normals"),
			("MSPGL", "MspGL", "Compute vertex normals internally")))
	export_groups = bpy.props.BoolProperty(name="Vertex groups", description="Export vertex groups and weights", default=False)

	def draw(self, context):
		col = self.layout.column()
		col.prop(self, "export_lines")
		col.prop(self, "compound")
		col.prop(self, "smoothing")
		col.prop(self, "export_groups")
		self.general_col = col

		self.layout.separator()

		col = self.layout.column()
		col.label("Triangle strips")
		col.prop(self, "use_strips")
		col.prop(self, "use_degen_tris")
		col.prop(self, "max_strip_len")

		self.layout.separator()

		col = self.layout.column()
		col.label("Texturing")
		col.prop(self, "export_uv")
		col.prop(self, "tbn_vecs")
		col.prop(self, "tbn_uvtex")
		self.texturing_col = col

		self.layout.separator()

		col = self.layout.column()
		col.label("Vertex cache")
		col.prop(self, "optimize_cache")
		col.prop(self, "cache_size")

class ExportMspGLMesh(bpy.types.Operator, ExportMspGLMeshBase):
	bl_idname = "export_mesh.mspgl_mesh"
	bl_label = "Export Msp GL mesh"

	filename_ext = ".mesh"

	def create_exporter(self):
		from .export_mesh import MeshExporter
		return MeshExporter()

class ExportMspGLObject(bpy.types.Operator, ExportMspGLMeshBase):
	bl_idname = "export_mesh.mspgl_object"
	bl_label = "Export Msp GL object"

	filename_ext = ".object"

	external_tech = bpy.props.BoolProperty(name="External technique", description="Use an external technique specified in the object's properties", default=True)

	textures = bpy.props.EnumProperty(name="Textures", description="Export textures", default="REF",
		items=(("NONE", "None", "Ignore textures"),
			("REF", "Referenced", "Reference external data"),
			("INLINE", "Inline", "Embed textures in the object")))
	material_tex = bpy.props.BoolProperty(name="Material texture", description="Generate a texture based on material colors", default=False)
	srgb_colors = bpy.props.BoolProperty(name="sRGB colors", description="Export material colors as sRGB instead of linear", default=True)

	separate_mesh = bpy.props.BoolProperty(name="Separate mesh", description="Write mesh data into a separate file", default=False)
	shared_mesh = bpy.props.BoolProperty(name="Shared mesh", description="Use mesh name for mesh file to enable sharing", default=True)
	separate_tech = bpy.props.BoolProperty(name="Separate technique", description="Write technique data into a separate file", default=False)
	shared_tech = bpy.props.BoolProperty(name="Shared technique", description="Use material name for technique file to enable sharing", default=True)

	export_lods = bpy.props.BoolProperty(name="Export LoDs", description="Export all levels of detail", default=True)

	def create_exporter(self):
		from .export_object import ObjectExporter
		return ObjectExporter()

	def draw(self, context):
		super().draw(context)

		col = self.general_col
		col.prop(self, "external_tech")
		col.prop(self, "export_lods")

		col = self.texturing_col
		col.prop(self, "textures")
		col.prop(self, "material_tex")
		col.prop(self, "srgb_colors")

		self.layout.separator();

		col = self.layout.column()
		col.label("Files")
		col.prop(self, "separate_mesh")
		if self.separate_mesh:
			col.prop(self, "shared_mesh")
		col.prop(self, "separate_tech")
		if self.separate_tech:
			col.prop(self, "shared_tech")

class ExportMspGLArmature(bpy.types.Operator, ExportMspGLBase):
	bl_idname = "export.mspgl_armature"
	bl_label = "Export Msp GL armature"

	filename_ext = ".arma"

	def create_exporter(self):
		from .export_armature import ArmatureExporter
		return ArmatureExporter()

class ExportMspGLScene(bpy.types.Operator, ExportMspGLBase):
	bl_idname = "export_scene.mspgl_scene"
	bl_label = "Export Msp GL scene"

	filename_ext = ".scene"

	external_tech = bpy.props.BoolProperty(name="External techniques", description="Use external techniques specified in objects' properties", default=True)
	resource_collection = bpy.props.BoolProperty(name="Resource collection", description="Put resources to a single collection file", default=True)

	def create_exporter(self):
		from .export_scene import SceneExporter
		return SceneExporter()

	def draw(self, context):
		col = self.layout.column()
		col.prop(self, "resource_collection")

def menu_func_export(self, context):
	self.layout.operator(ExportMspGLMesh.bl_idname, text="Msp GL mesh")
	self.layout.operator(ExportMspGLObject.bl_idname, text="Msp GL object")
	self.layout.operator(ExportMspGLArmature.bl_idname, text="Msp GL armature")
	self.layout.operator(ExportMspGLScene.bl_idname, text="Msp GL scene")

from .properties import MspGLMeshProperties, MspGLObjectProperties

def register():
	bpy.utils.register_module(__name__)

	bpy.types.INFO_MT_file_export.append(menu_func_export)

	from .properties import register_properties
	register_properties();

def unregister():
	bpy.utils.unregister_module(__name__)

	bpy.types.INFO_MT_file_export.remove(menu_func_export)

if __name__=="__main__":
	register()
